package vt.CS5934.SwitchRoom.controllers;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import vt.CS5934.SwitchRoom.models.ResponseModel;
import vt.CS5934.SwitchRoom.models.UserModel;
import vt.CS5934.SwitchRoom.services.UserService;

import java.security.NoSuchAlgorithmException;

/**
 * The "@RestController" made the class into rest handle class
 * The "@RequestMapping("example")" on the class level make it only react to url ".../example/..."
 */
@CrossOrigin
@RestController
@RequestMapping("user")
public class UserController {
    /**
     * Autowired is a Spring feature that it will create or looking for the existing object in memory.
     * It usually uses on Repository class, Service class, or some globe object in the class.
     */
    @Autowired
    UserService userService;

    /**
     * You can use logger.[trace,debug,info,warn,error]("messages") to log into file
     */
    private final Logger logger = LoggerFactory.getLogger(UserController.class);

    /**
     * This function will handle the post function and change the JSON body into data class
     * @param newUser the JSON input in the request body
     */
    @PostMapping("/newUser")
    public ResponseModel handlePost(@RequestBody UserModel newUser) throws NoSuchAlgorithmException {
        logger.info("You reached the handlePost() functions.");
        ResponseModel response = new ResponseModel();

        try{
            UserModel userModel = userService.addUserToDB(newUser.getUsername(), userService.hashPassword(newUser.getPassword()), newUser.getEmail(), newUser.getFirstname(), newUser.getLastname(), newUser.getGender());
            response.setMessage("Saved successfully");
            response.setStatus(HttpStatus.OK);
            response.setData(userModel);
            return response;
        }catch (Exception e){
            return new ResponseModel(
                    "Error happen",
                    HttpStatus.OK,
                    null);
        }
    }

    @PostMapping("/loginUser")
    public ResponseModel loginUser(@RequestBody UserModel user) throws NoSuchAlgorithmException {
        logger.info("You reached the handlePost() functions.");
        ResponseModel response = new ResponseModel();
        String inputPassword = userService.hashPassword(user.getPassword());
//        hash.get_SHA_1_SecurePassword(user.getPassword());

        try{
            UserModel existUser = userService.loginUser(user.getUsername());
            if (existUser != null && existUser.getPassword().equals(inputPassword)) {
                response.setMessage("Login in successfully");
                response.setStatus(HttpStatus.OK);
            } else {
                response.setMessage("Couldn't find an account matching the login info you entered");
                response.setStatus(HttpStatus.FORBIDDEN);
            }
            existUser.setPassword(null);
            response.setData(existUser);

//            UserModel existUser = userService.loginUser(user.getUsername());
//            response.setMessage("Login in successfully");
//            response.setStatus(HttpStatus.OK);
//            existUser.setPassword(null);
//            response.setData(existUser);

            return response;
        }catch (Exception e){
            return new ResponseModel(
                    "INTERNAL_SERVER_ERROR",
                    HttpStatus.INTERNAL_SERVER_ERROR,
                    null);
        }
    }
}
